/*******************************************************************************
rpc.h

Copyright (C) 2000 Simon Nieuviarts

The RPC interface.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version. 

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*******************************************************************************/
#ifndef __RPC_H__
#define __RPC_H__


#include <oskit/com.h>
#include <oskit/error.h>
#include <oskit/net/socket.h>
#include <oskit/com/stream.h>
#include <oskit/dev/osenv_mem.h>
#include <oskit/dev/osenv_timer.h>


#define RPC_BUF_LEN 1440
typedef struct oskit_rpc_t
{
    struct oskit_rpc_ops *ops ;
    oskit_stream_t *s ;
    oskit_socket_factory_t *sf ;
    oskit_osenv_mem_t *mem ;
    oskit_u32_t ref_counter ;
    char *localhost ;
    oskit_u32_t short_timeout ;
    oskit_u32_t long_timeout ;
    oskit_u32_t prog ;
    oskit_u32_t vers ;
    char *wptr ;
    char *rptr ;
    oskit_size_t rlen ;
    char call_buf[RPC_BUF_LEN] ;
    char reply_buf[RPC_BUF_LEN] ;
} oskit_rpc_t ;


struct oskit_rpc_ops {

    /*** COM-specified IUnknown interface operations ***/
    OSKIT_COMDECL_IUNKNOWN(oskit_rpc_t)

    OSKIT_COMDECL (*rpc_call_init)( oskit_rpc_t *rpc, oskit_u32_t proc ) ;
    OSKIT_COMDECL (*rpc_call_wxdr_int)( oskit_rpc_t *rpc, oskit_u32_t x ) ;
    OSKIT_COMDECL (*rpc_call_wxdr_opaque)( oskit_rpc_t *rpc, const void *buf,
                                           oskit_size_t l ) ;
    OSKIT_COMDECL (*rpc_call_wxdr_string)( oskit_rpc_t *rpc,
                                           const char *string ) ;
    OSKIT_COMDECL (*rpc_call_wxdr_data)( oskit_rpc_t *rpc, const void *buf,
                                         oskit_size_t l ) ;
    OSKIT_COMDECL (*rpc_reply_rxdr_int)( oskit_rpc_t *rpc, oskit_u32_t *x ) ;
    OSKIT_COMDECL (*rpc_reply_rxdr_opaque)( oskit_rpc_t *rpc, char **p,
                                            oskit_size_t *l ) ;
    OSKIT_COMDECL (*rpc_reply_rxdr_data)( oskit_rpc_t *rpc, void **p,
                                            oskit_size_t l ) ;
    OSKIT_COMDECL_U (*rpc_reply_len)( oskit_rpc_t *rpc ) ;
    OSKIT_COMDECL (*rpc_do_call)( oskit_rpc_t *rpc ) ;
    OSKIT_COMDECL (*rpc_connect)( oskit_rpc_t *rpc, const char *host,
                                  oskit_u32_t prog, oskit_u32_t vers ) ;
} ;


#define oskit_rpc_query( rpc, iid, out_ihandle ) \
    ( (rpc)->ops->query( (rpc), (iid), (out_ihandle) ) )
#define oskit_rpc_addref( rpc ) ( (rpc)->ops->addref( (rpc) ) )
#define oskit_rpc_release( rpc ) ( (rpc)->ops->release( (rpc) ) )
#define oskit_rpc_call_init( rpc, proc ) \
    ( (rpc)->ops->rpc_call_init( (rpc), (proc) ) )
#define oskit_rpc_call_wxdr_int( rpc, x ) \
    ( (rpc)->ops->rpc_call_wxdr_int( (rpc), (x) ) )
#define oskit_rpc_call_wxdr_opaque( rpc, buf, l ) \
    ( (rpc)->ops->rpc_call_wxdr_opaque( (rpc), (buf), (l) ) )
#define oskit_rpc_call_wxdr_string( rpc, string ) \
    ( (rpc)->ops->rpc_call_wxdr_string( (rpc), (string) ) )
#define oskit_rpc_call_wxdr_data( rpc, buf, l ) \
    ( (rpc)->ops->rpc_call_wxdr_data( (rpc), (buf), (l) ) )
#define oskit_rpc_reply_rxdr_int( rpc, x ) \
    ( (rpc)->ops->rpc_reply_rxdr_int( (rpc), (x) ) )
#define oskit_rpc_reply_rxdr_opaque( rpc, p, l ) \
    ( (rpc)->ops->rpc_reply_rxdr_opaque( (rpc), (p), (l) ) )
#define oskit_rpc_reply_rxdr_data( rpc, p, l ) \
    ( (rpc)->ops->rpc_reply_rxdr_data( (rpc), (p), (l) ) )
#define oskit_rpc_reply_len( rpc ) \
    ( (rpc)->ops->rpc_reply_len( (rpc) ) )
#define oskit_rpc_do_call( rpc ) \
    ( (rpc)->ops->rpc_do_call( (rpc) ) )
#define oskit_rpc_connect( rpc, host, prog, vers ) \
    ( (rpc)->ops->rpc_connect( (rpc), (host), (prog), (vers) ) )


extern const struct oskit_guid oskit_rpc_iid ;
#define OSKIT_RPC_IID OSKIT_GUID(0x38eb9530, 0xf34d, 0x6f51, \
				0x9e, 0xed, 0x00, 0x10, 0x4b, 0x42, 0xcd, 0x22)


/* Create an object to do RPC calls. */
oskit_error_t oskit_rpc_init( oskit_osenv_mem_t *mem,
                              oskit_osenv_timer_t *timer,
                              oskit_socket_factory_t *sf,
                              const char *localhost,
                              oskit_u32_t short_timeout,
                              oskit_u32_t long_timeout,
                              oskit_rpc_t **out_rpc ) ;


#endif
